/* Database creation, insertion, and population routines
 *   By Charles Shiflett
 *
 * set ts=4 									( 4 character tabs )
 */

#include "post.h"
#include "ieee.h"

 /* init_databases()
  *  Creates the default tables used for the IEEE database 
  */

void init_database() {

    /* Note: order is important */


	exec("create table person (who SERIAL primary key, ieee_num INTEGER, 
			 real_name TEXT, user_name TEXT, password TEXT, phone INTEGER,
			 email TEXT, unique (ieee_num), unique(user_name),
			 street TEXT, zip INTEGER, city TEXT, url TEXT, comments TEXT[], 
	         description TEXT, ds TIMESTAMP, 
	         CHECK (user_name is NULL or password is not null),
	         CHECK ((street is null and zip is null and city is null) or
	           (street is not null and zip is not null and city is not null)))");

	exec("create table inventory (item_num SERIAL primary key, type text, sub_type text,
			 who INTEGER references person, 
			 size TEXT, color TEXT, def_price NUMERIC(24,2), stock INTEGER, 
			 description TEXT, ds TIMESTAMP
			 CHECK (def_price > 0))");

	exec("create table order_form (order_num SERIAL primary key, 
			 paid TEXT, delivered TEXT, who INTEGER references person, account TEXT,
	         description TEXT, ds TIMESTAMP, CHECK (paid in ('0','1')))");

	exec("create table receipt (receipt_num SERIAL primary key, order_num INTEGER references order_form, 
	         item_num INTEGER references inventory, price NUMERIC(24,2), taxable INTEGER, 
	         description TEXT, ds TIMESTAMP)");

	exec("create table expense (expense_num SERIAL primary key, amount NUMERIC(24,2), 
	         account TEXT, who INTEGER references person, description TEXT, ds TIMESTAMP)");
}

/* Creates data to populate database 
 *
 *    For populating the database with fictitous data, we are primarily
 *    interested in the relations, and not the content. For that reason,
 *	  only a few fields are filled in.
 */
void populate() {
	int random_number, i;

	/* constants for populate */
	int number_inventory=50,
		number_order=500,
		number_receipt=1500,
		number_person=250,
		number_expense=100;

	char tmp1[50], tmp2[50], tmp3[50], null_string[1]="";

	receipt_type receipt;
	expense_type expense;
	order_type	 order;
	inventory_type inventory;
	persocom	 person;


	/* populate order: inventory, person, order, receipt, expense */

	/* inventory loop */

	for (i=0; i<number_inventory; i++) {

		sprintf(tmp1, "%d", random() % (number_inventory/4));
		inventory.type=inventory.sub_type=inventory.size=tmp1;

		sprintf(tmp2, "%d", i);
		inventory.color=tmp2;

		inventory.price=random() % (10000-100) + 100;
		inventory.stock=random() % 20 + 2;
		inventory.description=null_string;

		insert_inventory(&inventory);
	}

	/* person loop */

	for (i=0; i<number_person; i++) {
		sprintf(tmp1, "%d", random() % (number_person/4));
		person.real_name=tmp1;

		person.city=person.password=person.email=person.street=
		                         person.url=person.description=null_string;
		(int)person.user_name=person.zip=person.ieee_number=person.phone=0;

		insert_person(&person);
	}

	/* order loop */

	for (i=0; i <number_order; i++) {
		random_number=random();
		sprintf(tmp1,"%d", (random_number & 0x1) ? 0 : 1);
		sprintf(tmp2,"%d", (random_number & 0x2) ? 0 : 1);
		sprintf(tmp3,"Petty Cash");

		order.paid=tmp1;
		order.delivered=tmp2;
		order.account=tmp3;
		order.description=null_string;

		insert_order(&order, (random_number>>2)%number_person );

	}

	/* receipt loop */

	for (i=0; i<number_receipt; i++) {
		/* set tax to either one or zero, and set price to .01-100$ */
		receipt.tax=(receipt.price=random() % 10000 + 1) & 1;
		receipt.description=null_string;
		insert_receipt(&receipt, random() % number_order, 
		                         random() % number_inventory);
	}

	/* expense loop */

	for (i=0; i < number_expense; i++) {

		sprintf(tmp1,"Petty Cash");
		expense.account=tmp1;

		expense.description=null_string;
		expense.amount=random() % 100000 + 1000;

		insert_expense(&expense, random() % number_person);
	}

}

int main() {

	/* Initilize our random number generator based on where in memory we are */
	char memory_location;	
	srand((unsigned int)&memory_location);

	init_database();
/*	populate();*/

	return(0);
}
